#pragma once

class CConfig
{
public:
	class IChangeListener
	{
	public:
		virtual void ConfigChanged() = 0;
	};

	// Get singleton instance.
	static CConfig *GetConfig();
	// Get config value as string. This method can throw TBzrException.
	std::wstring GetString(std::wstring config_name);
	// Get config value as boolean. This method can throw TBzrException.
	BOOL GetBool(std::wstring config_name);

	void AddListener(IChangeListener *listener);
	void RemoveListener(IChangeListener *listener);

protected:
	CConfig(void);
	~CConfig(void);

	// Get config value from tbzrcache process.
	std::wstring RequestGetConfig(std::wstring config_name);
	// Thread proc which watches config change event.
	static void WatchConfigChange(void *param);

	std::map<std::wstring, std::wstring> m_cache;
	std::vector<IChangeListener *> m_listeners;
	HANDLE m_thread_stop_event;
	HANDLE m_config_changed_event;
	static CConfig *m_singleton;
	static CriticalSection m_lock;
};
