// IconOverlay.cpp : Implementation of CIconOverlay

#include "stdafx.h"
#include "IconOverlay.h"
#include "RemoteLink.h"
#include "Config.h"


// CIconOverlayBase

CriticalSection CIconOverlayBase::m_last_request_lock("IconOverlay::last_request_lock");
DWORD CIconOverlayBase::m_last_request_tick = 0;
std::wstring CIconOverlayBase::m_last_request_path;
DWORD CIconOverlayBase::m_last_request_value = 0;

CIconOverlayBase::CIconOverlayBase() : m_refcnt(0)
{
}

STDMETHODIMP CIconOverlayBase::QueryInterface(const IID &riid, void **ppv)
{
    if (!ppv)
        return E_POINTER;
    *ppv = NULL;

    if (IsEqualIID(riid, IID_IShellExtInit) || IsEqualIID(riid, IID_IUnknown)) {
        *ppv = (LPSHELLEXTINIT) this;
    }
    else if (IsEqualIID(riid, IID_IShellIconOverlayIdentifier)) {
        *ppv = (IShellIconOverlayIdentifier*) this;
    }
    if (*ppv) {
        AddRef();
        return NOERROR;
    }

    return E_NOINTERFACE;
}

STDMETHODIMP CIconOverlayBase::GetOverlayInfo(LPWSTR pwszIconFile, int cchMax, int * pIndex, DWORD * pdwFlags)
{
	// Note that TortoiseOverlays handles the icon itself.  This is where
	// we could record that an overlay has been loaded though, so we could
	// gracefully fall back when it is not.
	// ALSO: This is called before IsMemberOf.  It might be a good
	// opportunity to spin up a thread to maintain the pipe connection for
	// us?
	if (cchMax && pwszIconFile)
		*pwszIconFile = 0;
	if (pIndex)
		*pIndex = 0;
	if (pdwFlags)
		*pdwFlags = 0;
	return S_OK;
}

STDMETHODIMP CIconOverlayBase::IsMemberOf(LPCWSTR pwszPath, DWORD /*dwAttrib*/)
{
	try {
		if (!g_isExplorer && CConfig::GetConfig()->GetBool(L"icon_overlays.show_only_in_explorer"))
			return S_FALSE;
	}
    catch (TBzrException &e) {
		TBZR_ERROR("CConfig::GetBool failed. %s\n", e.what());
        return E_FAIL;
	}

	HRESULT rc = GetFileStatus(pwszPath)==GetMyState() ? S_OK : S_FALSE;
	TBZR_MUTTER("IsMemberOf state=%S, path='%S' - result=%s\r\n",
				GetMyStateName(), pwszPath, rc==S_OK ? "True" : "False");
	return rc;
}

STDMETHODIMP CIconOverlayBase::GetPriority(int * pIPriority)
{
	if (pIPriority)
		*pIPriority = 0;
	return S_OK;
}

DWORD CIconOverlayBase::GetFileStatus(LPCWSTR pwszPath)
{
	// As each of the overlay classes will generally be called one after
	// another for the same filename, we keep a quick cache of the last 
	// thing we were asked for.
	{ // scope for the lock.
		AutoCritical _autolock(m_last_request_lock);
		if (::GetTickCount() - m_last_request_tick < 2000 && 
			pwszPath &&
			wcscmp(pwszPath, m_last_request_path.c_str())==0)
			return m_last_request_value;
	}
	// need to hit the pipe.
	DWORD this_val;
	try {
		this_val = CRemoteLink::GetLink()->RequestFileStatus(pwszPath, 1);
	}
	catch (TBzrException &e) {
		TBZR_ERROR("RequestFileStatus failed. %s\n", e.what());
		this_val = STATE_NOTHING;
	}

	// update our mini-cache.
	{ // scope for the lock.
		AutoCritical _autolock(m_last_request_lock);
		m_last_request_tick = ::GetTickCount();
		m_last_request_path = pwszPath;
		m_last_request_value = this_val;
	}
	TBZR_MUTTER("Remote pipe said status of '%S' is %d\r\n", pwszPath, this_val);
	return this_val;
}
/* vim: set ts=4 sw=4 noexpandtab : */
