// IconOverlay.h : Declaration of the CIconOverlay

#pragma once
#include "resource.h"       // main symbols

// CIconOverlay

class CIconOverlayBase :
	public IShellIconOverlayIdentifier
{
protected:
	// From app.py:
    enum {
        STATE_NOTHING, 
        STATE_UNVERSIONED,
        STATE_IGNORED,
        STATE_UNCHANGED,
        STATE_PROGRESS,
        STATE_MODIFIED,
        STATE_ADDED,
        STATE_DELETED,
        STATE_MISSING,
        STATE_CONFLICT,
    };

	virtual DWORD GetMyState() = 0;
	virtual const TCHAR *GetMyStateName() = 0;

    LONG m_refcnt;

public:
    CIconOverlayBase();
    STDMETHODIMP QueryInterface(const IID &riid, void **ppv);

    TBZR_IUNKNOWN_IMPLEMENT();

    // IIconOverlay implementation
	STDMETHOD(IsMemberOf)(LPCWSTR pwszPath, DWORD dwAttrib);
	STDMETHOD(GetOverlayInfo)(LPWSTR pwszIconFile, int cchMax, int * pIndex, DWORD * pdwFlags);
	STDMETHOD(GetPriority)(int * pIPriority);
protected:
	DWORD CIconOverlayBase::GetFileStatus(LPCWSTR pwszPath);

	// As each of the overlay classes will generally be called one after
	// another for the same filename, we keep a quick cache of the last 
	// thing we were asked for.
    static CriticalSection m_last_request_lock;
	static DWORD m_last_request_tick;
	static std::wstring m_last_request_path;
	static DWORD m_last_request_value;

public:

};


#define MAKE_OVERLAY_CLASS(TYPE, STATE) \
class CIconOverlay##TYPE : \
	public CIconOverlayBase \
{ \
public: \
	virtual DWORD GetMyState() { return STATE; } \
	virtual const TCHAR *GetMyStateName() { return _T(#TYPE); } \
};

MAKE_OVERLAY_CLASS(Added, STATE_ADDED);
MAKE_OVERLAY_CLASS(Normal, STATE_UNCHANGED);
MAKE_OVERLAY_CLASS(Modified, STATE_MODIFIED);
MAKE_OVERLAY_CLASS(Conflict, STATE_CONFLICT);
//MAKE_OVERLAY_CLASS(Locked, STATE_LOCKED);
//MAKE_OVERLAY_CLASS(ReadOnly, STATE_READONLY);
MAKE_OVERLAY_CLASS(Deleted, STATE_DELETED);
MAKE_OVERLAY_CLASS(Ignored, STATE_IGNORED);
// MAKE_OVERLAY_CLASS(Invalid);
// Use Unversioned icon as Progress state.
MAKE_OVERLAY_CLASS(Unversioned, STATE_PROGRESS);
