// Simple TortoiseSVN-like Bazaar plugin for the Windows Shell
// Published under the GNU GPL, v2 or later.
// Copyright (C) 2008 Canonical
//
// Contributors:
//   Mark Hammond
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#include "stdafx.h"
#include "LinkBuffer.h"

unsigned char CLinkBufferManager::m_version[] = {0,1};

CLinkBufferManager::CLinkBufferManager()
{
	ResetForRequest((unsigned short)-1); // invalid value until known
}

void CLinkBufferManager::ResetForRequest(unsigned short request_type)
{
	m_buffer.req_header.ver_maj = m_version[0];
	m_buffer.req_header.ver_min = m_version[1];
	m_buffer.req_header.total_bytes = sizeof(m_buffer.req_header);
	m_buffer.req_header.request_type = request_type;
	m_read_pos = sizeof(m_buffer.resp_header);
}

void CLinkBufferManager::PushString(const wchar_t *s)
{
	// strings get their size first, then the unicode bytes.
	short cb = (short)wcslen(s) * sizeof(wchar_t);
	PushRaw(&cb, sizeof(cb));
	PushRaw(s, cb);
}

void CLinkBufferManager::PushInt(__int64 v)
{
	// integers are always pushed as 64bits - they might be a hwnd and
	// need to roundtrip via the remote process.
	PushRaw(&v, sizeof(v));
}

void CLinkBufferManager::PushSize(unsigned short v)
{
	// integers are always pushed as 64bits - they might be a hwnd and
	// need to roundtrip via the remote process.
	PushRaw(&v, sizeof(v));
}

void CLinkBufferManager::PushRaw(const void *bytes, unsigned short cb)
{
    if (m_buffer.req_header.total_bytes + cb >= sizeof(m_buffer.buffer)) {
        throw TBzrWin32Exception(ERROR_BUFFER_OVERFLOW,
            "CLinkBufferManager::PushRaw() bytes are too large.");
    }
	memcpy(m_buffer.buffer+m_buffer.req_header.total_bytes, bytes, cb);
	m_buffer.req_header.total_bytes += cb;
}

// "pop" values from the response.
void *CLinkBufferManager::PopRaw(unsigned short cb)
{
    if (m_read_pos + cb >= sizeof(m_buffer.buffer)) {
        throw TBzrWin32Exception(ERROR_BUFFER_OVERFLOW,
            "CLinkBufferManager::PopRaw() bytes are too large.");
    }

	void *pVal = ((BYTE *)m_buffer.buffer+m_read_pos);
	m_read_pos += cb;
	return pVal;
}
__int64 CLinkBufferManager::PopInt()
{
	return *(__int64 *)PopRaw(sizeof(__int64));
}

unsigned short CLinkBufferManager::PopSize()
{
	return *(unsigned short *)PopRaw(sizeof(unsigned short));
}


std::wstring CLinkBufferManager::PopString()
{
	unsigned short num_bytes = PopSize();
	std::wstring ret;
	wchar_t *strval = (wchar_t *)PopRaw(num_bytes);
	ret.assign(strval, num_bytes / sizeof(wchar_t));
	return ret;
}