// Simple TortoiseSVN-like Bazaar plugin for the Windows Shell
// Published under the GNU GPL, v2 or later.
// Copyright (C) 2008 Canonical
//
// Contributors:
//   Mark Hammond
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#pragma once

#pragma pack(push, 1) // need byte packing for the "wire" structures.

// The header we send in our requests.
struct ReqHeader
{
	unsigned short total_bytes;
	unsigned char ver_maj;
	unsigned char ver_min;
	unsigned short request_type;
};

// The header the remote process sends in its response.
struct RespHeader
{
	unsigned short total_bytes;
	unsigned short error_code;
};

// This is the request *and* response buffer.  It is filled
// before sending a request, then the response clobbers the
// data, ready to be parsed as the response.
struct ReqBuffer
{
	union {
		ReqHeader req_header;
		RespHeader resp_header;
		// like the python version, a fixed size buffer.
		BYTE buffer[1024*32];
	};
};

#pragma pack(pop)

class CLinkBufferManager
{
public:
	CLinkBufferManager();
	ReqBuffer m_buffer;
	void ResetForRequest(unsigned short request_type);

	// "push" values for the request.
	void PushString(const wchar_t *s);
	void PushString(std::wstring &s) {PushString(s.c_str());}
	// all integer 'values' are 64bits.
	void PushInt(__int64 v);
	// but all integer 'sizes' (eg, num children, strlens, etc) are shorts.
	void PushSize(unsigned short v);
	// convenience to push a vector<>'s 'size' result.
	void PushSize(size_t v) {PushSize((unsigned short)v);}

	// "pop" values from the response.
	__int64 CLinkBufferManager::PopInt();
	unsigned short CLinkBufferManager::PopSize();
	std::wstring PopString();

protected:
	void PushRaw(const void *bytes, unsigned short cb);
	void *PopRaw(unsigned short cb);

	// position in the buffer we are reading from.
	int m_read_pos;

	// This is the version of the "protocol"
	static unsigned char m_version[2];
};
