// Simple TortoiseSVN-like Bazaar plugin for the Windows Shell
// Published under the GNU GPL, v2 or later.
// Copyright (C) 2008 Canonical
//
// Contributors:
//   Mark Hammond
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#pragma once

#include "LinkBuffer.h"
#include "tbzrstructs.h"


class CRemoteLink
{
public:
	// This class is a singleton - here is the instance!
	static CRemoteLink *GetLink();
//	std::wstring RequestHello();
//	std::wstring RequestGoodbye();
	DWORD RequestFileStatus(const WCHAR *filename, DWORD flags);
	void RequestReleaseHandles(const WCHAR *path);
	std::vector<TBZRMenuInfo> RequestMenuCommands(std::vector<std::wstring>, DWORD);
	std::wstring RequestGetParam(const WCHAR *param_name);

private:
	CRemoteLink();
	~CRemoteLink();
	void ClosePipe();
	void EnsurePipeOpen();
	void StartRemoteProcess();
	std::wstring GetPipeName();
	void MakeRequest();
	void RawRequest();

	// an unnamed (=local) manual reset event for use in the overlapped structure
	// hevent lives longer than one instance of the pipe
	HANDLE m_hevent;
	HANDLE m_hpipe;
	// m_last_timeout is unfortunately named - it really means 'when does
    // the last timeout expire" - ie, at what time do I ertry the next
	// attempt at connecting/starting the process...
	DWORD m_last_timeout;
	CLinkBufferManager *m_buffer;
	static DWORD m_timeout_ms;
	static CriticalSection m_lock;
	static CRemoteLink *m_singleton;
};

