// tbzrshellext.cpp : Implementation of DLL Exports.


#include "stdafx.h"
#include "resource.h"
#include "CopyHook.h"
#include "IconOverlay.h"
#include "ContextMenu.h"
#include "RemoteLink.h"

#include <new>
#include <string>

using namespace std;

inline static std::string Guid2String(const GUID &g) {
    wchar_t wbuf[64];
    char buf[64];
    ::StringFromGUID2(g, wbuf, 64);
    for (int i = 0; i < 64; ++i) {
        buf[i] = (char)wbuf[i];
    }
    return std::string(buf);
}

static bool perUserRegistration = false;
HMODULE tbzr::g_hModule;
LONG tbzr::g_numObject;
BOOL tbzr::g_isExplorer;

/// Class Factory. This class is static singleton.
template<typename T>
class TBzrClassFactory: public IClassFactory
{
    LONG m_refcnt;
public:

    TBzrClassFactory() : m_refcnt(0) {}

    STDMETHODIMP QueryInterface(REFIID riid, LPVOID FAR *ppv) {
        if (!ppv)
            return E_POINTER;
        *ppv = NULL;
        if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory)) {
            *ppv = (LPCLASSFACTORY)this;
            AddRef();
            return NOERROR;
        }
        return E_NOINTERFACE;
    }

    STDMETHODIMP_(ULONG) AddRef() {
        InterlockedIncrement(&tbzr::g_numObject);
        return InterlockedIncrement(&m_refcnt);
    }
    STDMETHODIMP_(ULONG) Release() { // don't delete this.
        InterlockedDecrement(&tbzr::g_numObject);
        return InterlockedDecrement(&m_refcnt);
    }

    STDMETHODIMP CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID FAR* ppvObj) {
        if (pUnkOuter)
            return CLASS_E_NOAGGREGATION;

        T *obj = new T();
        HRESULT hr = obj->QueryInterface(riid, ppvObj);
        if (FAILED(hr))
            delete obj;
        return hr;
    }

    STDMETHODIMP LockServer(BOOL) {
        return NOERROR;
    }
};

// Used to determine whether the DLL can be unloaded by OLE
STDAPI DllCanUnloadNow(void)
{
    TBZR_MUTTER("DllCanUnloadNow - %d\n", tbzr::g_numObject);
    return tbzr::g_numObject == 0 ? S_OK : S_FALSE;
}


// Returns a class factory to create an object of the requested type
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    static TBzrClassFactory<CCopyHook> copyhookFactory;
    static TBzrClassFactory<CIconOverlayAdded> overlayAddedFactory;
    static TBzrClassFactory<CIconOverlayNormal> overlayNormalFactory;
    static TBzrClassFactory<CIconOverlayModified> overlayModifiedFactory;
    static TBzrClassFactory<CIconOverlayConflict> overlayConflictFactory;
    static TBzrClassFactory<CIconOverlayDeleted> overlayDeletedFactory;
    static TBzrClassFactory<CIconOverlayIgnored> overlayIgnoredFactory;
    static TBzrClassFactory<CIconOverlayUnversioned> overlayUnversinedFactory;
    static TBzrClassFactory<CContextMenu> contextmenuFactory;

    if (rclsid == CLSID_CopyHook)
        return copyhookFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayAdded)
        return overlayAddedFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayNormal)
        return overlayNormalFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayModified)
        return overlayModifiedFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayConflict)
        return overlayConflictFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayDeleted)
        return overlayDeletedFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayIgnored)
        return overlayIgnoredFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_IconOverlayUnversioned)
        return overlayUnversinedFactory.QueryInterface(riid, ppv);

    if (rclsid == CLSID_ContextMenu)
        return contextmenuFactory.QueryInterface(riid, ppv);

    return CLASS_E_CLASSNOTAVAILABLE;
}

namespace {

    const GUID GUIDs[] = {
        CLSID_CopyHook,
        CLSID_IconOverlayAdded,
        CLSID_IconOverlayNormal,
        CLSID_IconOverlayModified,
        CLSID_IconOverlayConflict,
        CLSID_IconOverlayDeleted,
        CLSID_IconOverlayIgnored,
        CLSID_IconOverlayUnversioned,
        CLSID_ContextMenu
    };
    const char APP_ID[] = "{77450df0-b5e7-11dd-bcc2-005056c00008}";
    const char APP_NAME[] = "tbzrshellext";


    void RegSetStringValue(HKEY root, const char *keyname, const char *valname, const char *val)
    {
        HKEY hkey;
        long err;

        err = RegCreateKeyA(root, keyname, &hkey);
        if (err)
            throw TBzrWin32Exception(err, "RegCreateKeyA failed.");

        err = RegSetValueExA(hkey, valname, 0, REG_SZ, (const BYTE*)val, (ULONG)strlen(val)+1);
        RegCloseKey(hkey);

        if (err)
            throw TBzrWin32Exception(err, "RegSetValueExA failed.");
    }
}

// DllRegisterServer - Adds entries to the system registry
STDAPI DllRegisterServer(void)
{
    OutputDebugStringA("TortoiseBZR: DllRegisterServer\n");
    // registers object, typelib and all interfaces in typelib

    char dllFileName[MAX_PATH+1];
    GetModuleFileNameA(g_hModule, dllFileName, MAX_PATH);

    static const char *descs[] = {
        "TortoiseBZR CopyHook Shell Extension",
        "TortoiseBZR IconOverlay Shell Extension: Added",
        "TortoiseBZR IconOverlay Shell Extension: Normal",
        "TortoiseBZR IconOverlay Shell Extension: Modified",
        "TortoiseBZR IconOverlay Shell Extension: Conflict",
        "TortoiseBZR IconOverlay Shell Extension: Deleted",
        "TortoiseBZR IconOverlay Shell Extension: Ignored",
        "TortoiseBZR IconOverlay Shell Extension: Unversioned",
        "TortoiseBZR ContextMenu Shell Extension"
    };

    const int NUM_GUID = sizeof(GUIDs)/sizeof(GUIDs[0]);

    try {
        const HKEY hkcr = HKEY_CLASSES_ROOT;
        // Register class id.
        for (int i = 0; i < NUM_GUID; ++i) {
            std::string subkey = "CLSID\\";
            subkey += Guid2String(GUIDs[i]);

            RegSetStringValue(hkcr, subkey.c_str(), 0, descs[i]);

            subkey += "\\InprocServer32";
            RegSetStringValue(hkcr, subkey.c_str(), 0, dllFileName);
            RegSetStringValue(hkcr, subkey.c_str(), "ThreadingModel", "Both");
        }

        // APP ID
        RegSetStringValue(hkcr, "AppID\\{77450df0-b5e7-11dd-bcc2-005056c00008}", 0, APP_NAME);
        RegSetStringValue(hkcr, "AppID\\tbzrshellext.dll", "AppID", APP_ID);

        // CopyHook
        RegSetStringValue(hkcr, "Directory\\shellex\\CopyHookHandlers\\TortoiseBazaar",
                0, Guid2String(CLSID_CopyHook).c_str());

        //IconOverlay
        {
            HKEY root = perUserRegistration ? HKEY_CURRENT_USER : HKEY_LOCAL_MACHINE;

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Added",
                    "BZR", Guid2String(CLSID_IconOverlayAdded).c_str());

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Normal",
                    "BZR", Guid2String(CLSID_IconOverlayNormal).c_str());

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Modified",
                    "BZR", Guid2String(CLSID_IconOverlayModified).c_str());

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Conflict",
                    "BZR", Guid2String(CLSID_IconOverlayConflict).c_str());

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Deleted",
                    "BZR", Guid2String(CLSID_IconOverlayDeleted).c_str());

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Ignored",
                    "BZR", Guid2String(CLSID_IconOverlayIgnored).c_str());

            RegSetStringValue(root, "Software\\TortoiseOverlays\\Unversioned",
                    "BZR", Guid2String(CLSID_IconOverlayUnversioned).c_str());
        }

        // ContextMenu
        {
            const string clsid = Guid2String(CLSID_ContextMenu).c_str();

            RegSetStringValue(hkcr, "*\\shellex\\ContextMenuHandlers\\TortoiseBazaar", 0, clsid.c_str());
            RegSetStringValue(hkcr, "Directory\\Background\\shellex\\ContextMenuHandlers\\TortoiseBazaar", 0, clsid.c_str());
            RegSetStringValue(hkcr, "Directory\\shellex\\ContextMenuHandlers\\TortoiseBazaar", 0, clsid.c_str());
            RegSetStringValue(hkcr, "Folder\\shellex\\ContextMenuHandlers\\TortoiseBazaar", 0, clsid.c_str());
        }

        // Approved list
        {
            RegSetStringValue(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved",
                    Guid2String(CLSID_ContextMenu).c_str(), "TortoiseBazaar Context Menu");
            RegSetStringValue(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved",
                    Guid2String(CLSID_CopyHook).c_str(), "TortoiseBazaar Copy Hook");
            // We doesn't need to register icon overlay here. Because it is used through TortoiseOverlay.
        }
        return S_OK;
    }
    catch (TBzrWin32Exception&) {
        return E_FAIL;
    }
}


// DllUnregisterServer - Removes entries from the system registry
STDAPI DllUnregisterServer(void)
{
    const int NUM_GUID = sizeof(GUIDs)/sizeof(GUIDs[0]);
    const HKEY hkcr = HKEY_CLASSES_ROOT;

    // class id.
    for (int i = 0; i < NUM_GUID; ++i) {
        string subkey = "CLSID\\";
        subkey += Guid2String(GUIDs[i]);
        string subkey2 = subkey + "\\InprocServer32";

        RegDeleteKeyA(hkcr, subkey2.c_str());
        RegDeleteKeyA(hkcr, subkey.c_str());
    }

    // APP ID
    RegDeleteKeyA(hkcr, "AppID\\{77450df0-b5e7-11dd-bcc2-005056c00008}");
    RegDeleteKeyA(hkcr, "AppID\\tbzrshellext.dll");

    // CopyHook
    RegDeleteKeyA(hkcr, "Directory\\shellex\\CopyHookHandlers\\TortoiseBazaar");

    {   //IconOverlay
        HKEY root = perUserRegistration ? HKEY_CURRENT_USER : HKEY_LOCAL_MACHINE;

        const char *keys[] = {
            "Software\\TortoiseOverlays\\Added",
            "Software\\TortoiseOverlays\\Normal",
            "Software\\TortoiseOverlays\\Modified",
            "Software\\TortoiseOverlays\\Conflict",
            "Software\\TortoiseOverlays\\Deleted",
            "Software\\TortoiseOverlays\\Ignored",
            "Software\\TortoiseOverlays\\Unversioned",
        };

        const int NUM_KEY = sizeof(keys) / sizeof(keys[0]);

        for (int i = 0; i < NUM_KEY; ++i) {
            HKEY hkey;
            long err = RegCreateKeyA(root, keys[i], &hkey);
            if (err) {
                continue;
            }
            RegDeleteValueA(hkey, "BZR");
            RegCloseKey(hkey);
        }
    }

    // ContextMenu
    RegDeleteKeyA(hkcr, "*\\shellex\\ContextMenuHandlers\\TortoiseBazaar");
    RegDeleteKeyA(hkcr, "Directory\\Background\\shellex\\ContextMenuHandlers\\TortoiseBazaar");
    RegDeleteKeyA(hkcr, "Directory\\shellex\\ContextMenuHandlers\\TortoiseBazaar");
    RegDeleteKeyA(hkcr, "Folder\\shellex\\ContextMenuHandlers\\TortoiseBazaar");

    // Approved list
    {
        HKEY hkey_approved;
        long err = RegOpenKeyA(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved",
                &hkey_approved);
        if (!err) {
            RegDeleteValueA(hkey_approved, Guid2String(CLSID_ContextMenu).c_str());
            RegDeleteValueA(hkey_approved, Guid2String(CLSID_CopyHook).c_str());
            RegCloseKey(hkey_approved);
        }
    }
    return S_OK;
}

// DllInstall - Adds/Removes entries to the system registry per user
//              per machine.
STDAPI DllInstall(BOOL bInstall, LPCWSTR pszCmdLine)
{
    HRESULT hr = E_FAIL;
    static const wchar_t szUserSwitch[] = L"user";

    if (pszCmdLine) {
    	if (_wcsnicmp(pszCmdLine, szUserSwitch, _countof(szUserSwitch)) == 0) {
            perUserRegistration = true;
        }
    }

    if (bInstall) {	
    	hr = DllRegisterServer();
    	if (FAILED(hr)) {
    		DllUnregisterServer();
    	}
    }
    else {
    	hr = DllUnregisterServer();
    }

    return hr;
}

extern "C" BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if(dwReason == DLL_PROCESS_ATTACH) {
        g_hModule = hInstance;

		WCHAR exeFilePath[MAX_PATH+1];
		WCHAR *cp;
	    GetModuleFileNameW(NULL, exeFilePath, MAX_PATH);
		cp = _tcsrchr(exeFilePath, '\\');
		if (_wcsicmp((cp ? cp + 1 : exeFilePath), L"explorer.exe") == 0)
			g_isExplorer = TRUE;
		else
			g_isExplorer = FALSE;
    }
    return TRUE;
}
