import sys
import logging
import codecs
from PyQt4 import QtCore, QtGui

from bzrlib.plugins.qbzr.lib.util import (
    QBzrDialog,
    open_browser,
    )

from tbzrcommands.ui.about import Ui_AboutForm
from tbzrlib.i18n import gettext
from tbzrlib.app import get_app
from cStringIO import StringIO
# from bzrlib.version:
def _get_tbzr_source_tree():
    import bzrlib
    from bzrlib import bzrdir, errors
    try:
        control = bzrdir.BzrDir.open_containing(__file__)[0]
        return control.open_workingtree(recommend_upgrade=False)
    except (errors.NotBranchError, errors.UnknownFormatError,
            errors.NoWorkingTree):
        return None

def _line_to_html(line):
    if not line:
        return "<br/>"
    ret_bits = []
    for i, ch in enumerate(line):
        if ch==' ':
            ret_bits.append("&nbsp;")
        else:
            break
    ret_bits.append(line[i:].rstrip())
    ret_bits.append("<br/>")
    return ''.join(ret_bits)

def safe_get_html(func, html):
    try:
        func(html)
    except:
        logging.getLogger("bzr").exception("Failed to generate HTML via %s",
                                           func)
        # don't bother writing error details to the html...

# An "About" dialog
class AboutWindow(QBzrDialog):
    _window_name = "tbzr-about"
    def __init__(self):
        title = [gettext("About Tortoise Bazaar")]
        super(AboutWindow, self).__init__(title)

        self.ui = Ui_AboutForm()
        self.ui.setupUi(self)

        # *sob* - again I failed to hook these up using QtDesigner...
        for widget in [self.ui.link_bazaar, self.ui.link_tbzr,
                       self.ui.link_tbzr_answers]:
            self.connect(widget, QtCore.SIGNAL("linkActivated(const QString &)"),
                         self.link_activated)

        html = []
        safe_get_html(self._get_about_tbzr, html)
        safe_get_html(self._get_about_bzr, html)
        safe_get_html(self._get_about_plugins, html)
        self.ui.textBrowser.setHtml(''.join(html))

    def _get_about_tbzr(self, html):
        import tbzrlib
        emit = html.append
        emit("<h3>" + gettext("About Tortoise Bazaar") + "</h3>")
        emit("Tortoise Bazaar %s<br/>" % tbzrlib.__version__)
        html.append("Executable: %s<br/>" % sys.executable)
        if getattr(sys, "frozen", None) is None:
            emit("Script: %s<br/>" % sys.argv[0])
        emit("Library: %s<br/>" % tbzrlib.__path__[0])
        # is tbzrlib itself in a branch?
        src_tree = _get_tbzr_source_tree()
        if src_tree:
            src_revision_id = src_tree.last_revision()
            revno = src_tree.branch.revision_id_to_revno(src_revision_id)
            emit(_line_to_html("from bzr checkout %s\n" % (src_tree.basedir,)))
            emit(_line_to_html("  revision: %s\n" % (revno,)))
            emit(_line_to_html("  revid: %s\n" % (src_revision_id,)))
            emit(_line_to_html("  branch nick: %s\n" % (src_tree.branch.nick,)))
            emit("<br/>")

        emit("%s" % tbzrlib.__copyright__)

    def _get_about_bzr(self, html):
        html.append("<h3>" + gettext("About Bazaar") + "</h3>")

        # hrmph - how do I get a "file-like" object which just gives back
        # unicode?  For now we go to-and-from utf8, ensuring we pass unicode
        # to Qt.
        to_file=codecs.getwriter("utf8")(StringIO())
        from bzrlib.version import show_version
        show_version(to_file=to_file)
        written = to_file.stream.getvalue().rstrip().decode("utf8")
        for line in written.splitlines():
            html.append(_line_to_html(line))

    def _get_about_plugins(self, html):
        html.append("<h3>" + gettext("About Plugins") + "</h3>")
        # plugins - duplicated from bzrlib.builtins
        import bzrlib.plugin
        from inspect import getdoc
        bzrlib.plugin.load_plugins()
        result = []
        for name, plugin in bzrlib.plugin.plugins().items():
            version = plugin.__version__
            if version == 'unknown':
                version = ''
            name_ver = '%s %s' % (name, version)
            d = getdoc(plugin.module)
            if d:
                doc = d.split('\n')[0]
            else:
                doc = '(no description)'
            result.append((name_ver, doc, plugin.path()))
        for name_ver, doc, path in sorted(result):
            html.append("<p><b>" + name_ver + "</b><br/>" + doc + "</p>")
        

    def link_activated(self, target):
        open_browser(target)

class cmd_about:
    def run(self):
        app = QtGui.QApplication(sys.argv)
        window = AboutWindow()
        window.show()
        app.exec_()
