# Copyright (C) 2008 Canonical
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os
from tbzrcommands import error_dialog
from bzrlib import errors
from cStringIO import StringIO

class cmd_remove(object):
    def run(self, file_list):
        from bzrlib.builtins import cmd_remove, tree_files
        assert file_list, "caller should handle empty list!"
        try:
            # We need to handle some special cases here:
            # * If you pass the root of a working tree, bzr silently ignores
            #   the directory.
            tree, file_list_in_tree = tree_files(file_list)
            for filename in file_list_in_tree:
                abspath = tree.abspath(filename)
                filename = tree.relpath(abspath)
                if not filename:
                    error_dialog("You can't use this command to delete the "
                                 "root of a working tree")
                    return
            del tree, file_list_in_tree

            cmd = cmd_remove()
            # XXX - this '.outf' handling is suspect, and nothing is written
            # to it!  Theoretically we could parse the output for the 'deleted'
            # lines (but we would still be left with the complication of not
            # knowing if it was a file or a dir before removal)
            cmd.outf = StringIO()
            cmd.run_direct(file_list=file_list, file_deletion_strategy='keep')
        except errors.BzrRemoveChangedFilesError, exc:
            # The default error message says to use '--force' - but we don't
            # expose that.  So we use a custom message
            fmt = ("Can't safely remove modified or unknown files:\n"
                   "%(changes_as_text)s"
                   "Please revert any modified files before removing them.")
            msg = fmt % exc.__dict__
            error_dialog(msg)
        except errors.BzrError, exc:
            error_dialog(str(exc))
