import sys
from PyQt4 import QtCore, QtGui

from bzrlib.plugins.qbzr.lib.util import (
    QBzrDialog,
    )

from tbzrcommands.ui.settings import Ui_SettingsForm
from tbzrlib.i18n import (gettext, N_)
from tbzrlib.app import get_app

class PrefItem:
    def __init__(self, pref_name):
        self.pref_name = pref_name
        self.config = get_app().config
        self.pref_ob = self.config.get_opt(pref_name)

    # Add widget to specified QGridLayout. QGridLayout has 2 columns.
    # 1st is for label, 2nd is for input box. 
    def add_to(self, grid, rowno):
        pass

    # Get new value from widget.
    def get_value(self):
        pass

    def apply(self):
        self.config.set(self.pref_name, self.get_value())

class CheckBoxPref(PrefItem):
    
    def add_to(self, parent, grid, rowno):
        cb = QtGui.QCheckBox(parent)

        val = self.config.get(self.pref_name)
        if val:
            cb.setCheckState(QtCore.Qt.Checked)
        else:
            cb.setCheckState(QtCore.Qt.Unchecked)

        cb.setText(gettext(self.pref_ob.doc))
        help = self.pref_ob.help_text
        if help:
            cb.setToolTip(gettext(help))

        grid.addWidget(cb, rowno, 0, 1, -1)

        self.cb = cb

    def get_value(self):
        return (self.cb.checkState() == QtCore.Qt.Checked)

class TextPref(PrefItem):

    def __init__(self, pref_name, multiline = False, height = 48):
        PrefItem.__init__(self, pref_name)
        self.multiline = multiline
        self.height = height

    def add_to(self, parent, grid, rowno):
        if self.multiline:
            edt = QtGui.QTextEdit(parent)
            edt.setLineWrapMode(QtGui.QTextEdit.NoWrap)
            edt.setAcceptRichText(False)
            edt.setMaximumHeight(self.height)
            edt.setTabChangesFocus(True)
        else:
            edt = QtGui.QLineEdit(parent)

        lbl = QtGui.QLabel(parent)
        val = self.config.get(self.pref_name)
        if val:
            edt.setText(val)
        lbl.setText(gettext(self.pref_ob.doc))

        help = self.pref_ob.help_text
        if help:
            edt.setToolTip(gettext(help))
            lbl.setToolTip(gettext(help))

        grid.addWidget(lbl, rowno, 0, 1, 1)
        grid.addWidget(edt, rowno, 1, 1, 1)

        self.edt = edt

    def get_value(self):
        if self.multiline:
            return self.edt.toPlainText()
        else:
            return self.edt.text()

# Generic 'panel' tied to the tree widget.
class PrefPanel(QtGui.QWidget):
    def __init__(self, parent):
        QtGui.QWidget.__init__(self, parent)
        self.centralwidget = QtGui.QVBoxLayout(self)
        self.setup()

    def addGroupBoxAndLayout(self, text, klass = QtGui.QVBoxLayout):
        gb = QtGui.QGroupBox(gettext(text), self)
        self.centralwidget.addWidget(gb)
        layout = klass(gb)
        return layout

    def setup(self):
        pass

    def apply(self):
        pass

# A panel that knows about our config object.
class ConfigPanel(PrefPanel):
    # 'connect' a bool preference with a checkbox. Keep the 'connection' with
    # the checkbox separate from the creation of the checkbox, incase we use
    # QtDesigner based forms.
    
    def __init__(self, parent):
        self.apply_handlers = []
        PrefPanel.__init__(self, parent)

    def apply(self):
        for handler in self.apply_handlers:
            handler()

    # XXX Obsolete
    def connect_checkbox_to_pref(self, cb, pref_name):
        config = get_app().config
        pref_ob = config.get_opt(pref_name)
        val = config.get(pref_name)
        if val:
            cb.setCheckState(QtCore.Qt.Checked)
        else:
            cb.setCheckState(QtCore.Qt.Unchecked)

        cb.setText(gettext(pref_ob.doc))
        help = pref_ob.help_text
        if help:
            cb.setToolTip(gettext(help))
        # the event handler.
        def cb_changed(state, cb=cb, pref_name=pref_name, config=config):
            new_val = state == QtCore.Qt.Checked
            config.set(pref_name, new_val)

        self.parentWidget().connect(cb, QtCore.SIGNAL("stateChanged(int)"),
                                    cb_changed)

    # XXX Obsolete
    # Create a groupbox filled with boolean checkboxes.
    def create_groupbox_of_bool_prefs(self, gb_title, *pref_names):
        gb = self.addGroupBoxAndLayout(gb_title)
        for name in pref_names:
            cb = QtGui.QCheckBox(self)
            self.connect_checkbox_to_pref(cb, name)
            gb.addWidget(cb)
        return gb

    def create_groupbox_of_prefs(self, gb_title, *pref_items):
        gb = self.addGroupBoxAndLayout(gb_title, QtGui.QGridLayout)
        gb.setColumnStretch(0, 0)
        for idx, item in enumerate(pref_items):
            gb.setRowStretch(idx, 0)
            item.add_to(self, gb, idx)
            self.apply_handlers.append(item.apply)

# Implementation classes for specific options...
class GeneralPrefPanel(ConfigPanel):
    def setup(self):
        self.create_groupbox_of_prefs(N_("Taskbar"),
                CheckBoxPref('task_bar.always_show_icon'))

        # Create a push-button to launch 'qconfig'
        def handler():
            from bzrlib.plugins.qbzr.lib.config import QBzrConfigWindow
            dlg = QBzrConfigWindow()
            dlg.exec_()

        button = QtGui.QPushButton(gettext("General Bazaar Options..."), self)
        self.parentWidget().connect(button, QtCore.SIGNAL("clicked()"),
                                    handler)

        # Add the button to the form - must put it in a VBox to prevent it
        # stretching to the full width of the window.
        layout = QtGui.QHBoxLayout(self)
        layout.addWidget(button)
        layout.addStretch(1)

        self.centralwidget.addLayout(layout)

        self.centralwidget.addStretch(1)

class OverlaysPrefPanel(ConfigPanel):
    def setup(self):
        self.create_groupbox_of_prefs(N_("Icon Overlays"),
                CheckBoxPref('icon_overlays.show_ignored'),
                CheckBoxPref('icon_overlays.show_as_mod_if_unversioned'),
                CheckBoxPref('icon_overlays.show_only_in_explorer'))

        self.create_groupbox_of_prefs(N_("Drive Types"),
                CheckBoxPref('queried_drives.removable'),
                CheckBoxPref('queried_drives.floppy'),
                CheckBoxPref('queried_drives.cdrom'),
                CheckBoxPref('queried_drives.fixed'),
                CheckBoxPref('queried_drives.remote'),
                CheckBoxPref('queried_drives.unknown'))

        self.centralwidget.addStretch(1)

class WhiteBlackPanel(ConfigPanel):
    def setup(self):
        self.create_groupbox_of_prefs(N_("Whitelist / Blacklist"),
                TextPref('icon_overlays.include_path', multiline=True, height=96),
                TextPref('icon_overlays.exclude_path', multiline=True, height=96)
                )

        self.centralwidget.addStretch(1)


def get_ui_definition():
    return [
        # Name of tree widget item, class for prefs, child definitions.
        (N_("General Options"), GeneralPrefPanel, []),
        (N_("Icon Overlays"), OverlaysPrefPanel, [
            (N_("Whitelist / Blacklist"), WhiteBlackPanel, []),
            ],),
    ]

# The main, top-level settings window.  Holds the tree and the 'stacked'
# panel for the preferences.
class SettingsWindow(QBzrDialog):
    _window_name = "tbzr-settings"
    def __init__(self):
        title = [gettext("Tortoise Bazaar Settings")]
        super(SettingsWindow, self).__init__(title)

        self.ui = Ui_SettingsForm()
        self.ui.setupUi(self)

        # clear the 'sample' items in the stacked object.
        stacked = self.ui.stacked
        while stacked.count():
            stacked.removeWidget(stacked.widget(0))

        # add the panels.
        self._panels = []
        self._create_tree_children(self.ui.tree.invisibleRootItem(),
                                   get_ui_definition())

        self.connect(self.ui.tree,
                     QtCore.SIGNAL("currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)"),
                     self.update_visible_panel)
        # *sob* and I can't work out how to connect the 'apply' button
        # magically in QtDesigner like 'accept' and 'reject' are...
        self.connect(self.ui.buttonBox.button(self.ui.buttonBox.Apply),
                     QtCore.SIGNAL("clicked()"),
                     self.apply)
                     

    def _create_tree_children(self, root, child_defs):
        stacked = self.ui.stacked
        for name, klass, children in child_defs:
            item = QtGui.QTreeWidgetItem(root)
            item.setText(0, gettext(name))
            # create the widget with the options.
            panel = klass(self)
            self._panels.append(panel)
            index = stacked.addWidget(panel)
            item.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(index))
            item.setExpanded(True)
            # And the children.
            self._create_tree_children(item, children)

    def update_visible_panel(self, item, prev):
        vindex = item.data(0, QtCore.Qt.UserRole)
        int_index, valid = vindex.toInt()
        self.ui.stacked.setCurrentIndex(int_index)

    def accept(self):
        self.apply()
        # super(SettingsWindow, self).accept() don't work with PyQt4.4
        QBzrDialog.accept(self)

    def apply(self):
        for panel in self._panels:
            panel.apply()
        get_app().config.write()

class cmd_settings:
    def run(self):

        app = QtGui.QApplication(sys.argv)

        window = SettingsWindow()
        window.show()
        app.exec_()
