import logging
logger = logging.getLogger('bzr.tbzr.cache')
mutter = logger.debug # for now...

from time import time, sleep
from threading import Thread
from Queue import Queue, Empty
from shellupdater import ShellUpdater

__all__ = ["Crawler"]

class Crawler(object):
    """Check status for directories asynchronously."""

    def __init__(self, wtcache):
        self.wtcache = wtcache
        self.shell_updater = ShellUpdater()
        self.start()

    def start(self):
        mutter("crawler starting.")
        self._queue = Queue()
        self.stopping = False

        self._worker = Thread(target=self._status_updater,
                              name="Status Checker",
                              )
        self._worker.daemon = True
        self._worker.start()

    def _status_updater(self):
        while True:
            try:
                sleep(1)
                if self.stopping:
                    break
                entry = self._queue.get()
                if entry is None or self.stopping:
                    break
                entries = {entry.path: entry}

                try:
                    while True:
                        entry = self._queue.get_nowait()
                        if entry is None:
                            break
                        entries[entry.path] = entry #ignore old entries.
                except Empty:
                    pass
                if self.stopping:
                    break

                for entry in entries.itervalues():
                    try:
                        path = entry.path.path
                        mutter("Checking %r", path)
                        entry.check_state()
                        self.shell_updater.update(path)
                    except Exception, e:
                        if entry.path in self.wtcache.dircache:
                            del self.wtcache.dircache[entry.path]
                        logger.warn(u"Status check for %s failed. (%s)", entry.path, e)
            except Exception, e:
                logger.error(u"Something occured on crawler: %s", e)

    def add_entry(self, wt_entry):
        logger.debug("Crawler.add_entry: %s", wt_entry.path)
        self._queue.put(wt_entry)

    def stop(self):
        mutter("crawler stopping")
        self.stopping = True
        self._queue.put(None)
