# coding: utf-8

"""Manage set of paths"""

from bisect import bisect, bisect_left
from vcspath import VCSPath

class PathSet(list):
    """Manage set of paths."""

    def add(self, vcspath):
        assert isinstance(vcspath, VCSPath)
        vcspath = vcspath.long_path()

        index = bisect_left(self, vcspath)
        if len(self) != index and self[index] == vcspath:
            return
        self.insert(index, vcspath)

    def is_contained(self, maybe_child):
        return len(self.get_container(maybe_child)) != 0

    def get_container(self, maybe_child):
        if not self:
            return []

        child = maybe_child.long_path()

        index = bisect(self, child)
        r = []
        while index:
            index -= 1
            if self[index].is_ancestor_of(child):
                r.append(self[index])
            else:
                break
        return r

    def __delitem__(self, delpath):
        i = bisect_left(self, delpath)
        if i == len(self) or self[i] != delpath:
            return
        list.__delitem__(self, i)

    def __repr__(self):
        s = ["PathSet(["]
        for p in self:
            s.append(repr(p))
            s.append(', ')
        s.append("])")
        return ''.join(s)


if __name__ == '__main__':
    # TODO: make more tests.
    t = PathSet()
    assert not t.is_contained(VCSPath(u"C:\\foo"))
    t.add(VCSPath(u"c:\\foo\\bar"))
    assert t.is_contained(VCSPath(u"C:\\Foo\\bar"))
    assert t.is_contained(VCSPath(u"C:\\foo\\bar\\baz"))
    assert t.is_contained(VCSPath(u"C:\\foo\\bar\\baz\\foo"))
    assert not t.is_contained(VCSPath(u"C:\\bar\\baz\\foo"))
    t.add(VCSPath(u"C:\\Foo\\Bar"))
    assert len(t) == 1
    t.add(VCSPath(u"C:\\foO\\bz"))
    assert len(t) == 2

    t = PathSet()
    t.add(VCSPath(u"C:\\foo"))
    t.add(VCSPath(u"C:\\bar"))
    t.add(VCSPath(u"C:\\bar\\baz"))
    c = t.get_container(VCSPath(u"C:\\bar\\baz"))
    assert len(c) == 2
