# This should probably be re-factored to make the communications mechanism
# discrete from the implemention of the request/response, but that can
# come later once we have more of a clue

import time
import logging
import threading
import struct

import pywintypes
import win32event
import win32pipe
import win32file
import win32con
import winerror

logger = logging.getLogger("bzr.tbzr.pipe")

from tbzrlib.pipe import get_pipe_name

class PipeServer(threading.Thread):
    def __init__(self, app):
        self.app = app
        self.running_threads = []
        threading.Thread.__init__(self)

    def run(self):
        logger.debug("PipeServer thread starting")
        try:
            try:
                self.run_connection_loop()
            except:
                logger.exception("PipeServer failed")
            
        finally:
            logger.debug("PipeServer thread stopped")
            self.app.stop()

    def run_connection_loop(self):
        open_mode = win32con.PIPE_ACCESS_DUPLEX| win32con.FILE_FLAG_OVERLAPPED
        pipe_mode = win32con.PIPE_TYPE_MESSAGE | \
                    win32con.PIPE_READMODE_MESSAGE | win32con.PIPE_WAIT
        max_instances = win32con.PIPE_UNLIMITED_INSTANCES
        overlapped = pywintypes.OVERLAPPED()
        overlapped.hEvent = win32event.CreateEvent(None,0,0,None)
        name = get_pipe_name()
        while not self.app.is_stopping():
            logger.debug("Creating NamedPipe %r", name)
            try:
                handle = win32pipe.CreateNamedPipe(name, open_mode, pipe_mode,
                            max_instances, 0, 0, 0, None)
            except win32pipe.error, details:
                logger.error("Error creating pipe: %s", details)
                time.sleep(1)
                continue
            try:
                hr = win32pipe.ConnectNamedPipe(handle, overlapped)
            except win32pipe.error, details:
                logger.error("Error connecting to pipe: %s", details)
                handle.Close()
                time.sleep(1)
                continue

            if hr==winerror.ERROR_PIPE_CONNECTED:
                # Client is already connected - signal event
                win32event.SetEvent(overlapped.hEvent)
            # Wait for either a connection or a shutdown request.
            rc = win32event.WaitForMultipleObjects(
                                [self.app.stop_event, overlapped.hEvent], False,
                                win32event.INFINITE)

            if rc == win32event.WAIT_OBJECT_0:
                break # stopping
            # start a new thread to handle the connection.
            t = PipeConnectionHandler(self, handle)
            t.start()
            # and back around waiting for another connection

class PipeConnectionHandler(threading.Thread):
    def __init__(self, server, handle):
        self.server = server
        self.handle = handle
        threading.Thread.__init__(self)

    def run(self):
        self.server.running_threads.append(self)
        try:
            logger.debug("Connection thread starting")
            try:
                try:
                    self.run_command_loop()
                except:
                    logger.exception("PipeConnectionHandler failed.")
            finally:
                logger.debug("Connection thread stopping")
        finally:
            self.server.running_threads.remove(self)

    def run_command_loop(self):
        handle = self.handle
        self.handle = None # handle should now auto-close before the instance
        overlapped = pywintypes.OVERLAPPED()
        overlapped.hEvent = win32event.CreateEvent(None,0,0,None)
        app = self.server.app
        # allocate a static buffer for the commands to avoid heap thrashing.
        buf = win32file.AllocateReadBuffer(1024)
        stop_event = app.stop_event
        cmd_dispatcher = app.command_dispatcher

        while not self.server.app.is_stopping():
            try:
                hr, _ = win32file.ReadFile(handle, buf, overlapped)
                rc = win32event.WaitForMultipleObjects(
                            [stop_event, overlapped.hEvent], False,
                            win32event.INFINITE)
                if rc == win32event.WAIT_OBJECT_0:
                    break # stopping
            except win32file.error, exc:
                # almost certainly just the client disconnecting or otherwise
                # misbehaving.
                if exc[0]==winerror.ERROR_BROKEN_PIPE:
                    logger.info("Closing thread connection due to client disconnection.")
                else:
                    logger.info("Closing thread connection due to client error: %s",
                                exc)
                break
            result = ''
            rc = 0
            try: # handle exceptions processing the request.
                nbytes = win32file.GetOverlappedResult(handle, overlapped, True)
                app.on_activity()
                result = cmd_dispatcher.dispatch(buf[:nbytes])
            except cmd_dispatcher.RequestFailed, exc:
                logger.warning("Request failed: %s", exc)
                rc = winerror.ERROR_INVALID_FLAGS
            except win32file.error, exc:
                if exc[0] == winerror.ERROR_BROKEN_PIPE:
                    logger.info("Client disconnected from pipe - closing connection")
                else:
                    # failed to read a full request - not our problem (but log it)
                    logger.info("Failed to read client request: %s", exc)
                # But this connection is dead either way
                break
            except:
                logger.exception("Error processing remote request")
                rc = winerror.ERROR_CAN_NOT_COMPLETE

            try:
                # Write the result
                hdr = struct.pack("HH", len(result) + struct.calcsize("HH"), rc)
                to_write = hdr + result
                logger.debug("Writing %d bytes to the pipe", len(to_write))
                rc, nbytes = win32file.WriteFile(handle, hdr + result)
            except win32file.error, exc:
                # almost certainly related to the client, not us!
                logger.info("Failed to write response to client: %s", exc)
                # should we terminate here?  Yes, as our "protocol" can't
                # possibly recover if some data didn't arrive..
                break

            # and loop again waiting for the next request
