"""Refresh icons in Explorer.

See TortoiseSVN's tsvncache/ShellUpdater.cpp
"""
import os
import logging

from win32com.shell import shell, shellcon

_log = logging.getLogger("bzr.tbzr.shellupdater")

__all__ = ("ShellUpdater",)

class ShellUpdater(object):

    _flushcount = 0

    def update(self, path):
        """update path's status asynchronous."""
        assert isinstance(path, unicode), "path must be unicode."
        _log.debug("Updating explorer: %r", path)
        try:
            if os.path.isdir(path):
                self._recursive_update(path)
            else:
                self._update_item(path, True)
        except Exception, e:
            _log.error("Exeption raised in ShellUpdater: %s", e)

    def _recursive_update(self, path):
        for root, dirs, files in os.walk(path, topdown=False):
            # To update directory, an item in directory must be update too.
            if not dirs and files:
                f = files[0]
                fp = os.path.join(root, f)
                self._update_item(fp)
            self._update_item(root)
        self._update_item(path, True)

    def _update_item(self, path, wait=False):
        """Update path's icon"""
        # buff size. I can't find docs about this number.
        # I found shellupdater fails to update icon sometimes when buffsize=64.
        if wait or self._flushcount > 48:
            flag = shellcon.SHCNF_FLUSH
            self._flushcount = 0
        else:
            flag = 0
            self._flushcount += 1

        flag |= shellcon.SHCNF_PATHW
        shell.SHChangeNotify(shellcon.SHCNE_UPDATEITEM, flag, path, None)

