# Copyright (C) 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import tempfile
from StringIO import StringIO

import pythoncom
from win32com.shell import shell, shellcon
import win32gui
import win32gui_struct
import win32con

from bzrlib.revisionspec import RevisionSpec
from bzrlib.tests import TestCaseWithTransport, TestCase

from tbzrlib.tests.shellext import get_clsid_for_handler, _ShellExtensionFeature

def get_pidl_for_dir(dirname):
    # Get an IShellFolder instance for the given path.
    pidl, attr = shell.SHILCreateFromPath(dirname,
                                          win32con.FILE_ATTRIBUTE_DIRECTORY)
    return pidl
    
def get_isf_for_pidl(pidl):
    folder = shell.SHGetDesktopFolder()
    for p in pidl[:-1]:
        folder = folder.BindToObject([p], None, shell.IID_IShellFolder)
    return folder

ContextMenuShellExtensionFeature = _ShellExtensionFeature('ContextMenu')

# XXX - should use multiply_tests_from_modules, but its not clear how the loader
# etc fits here yet.
class TestContextNonVersioned(TestCase):
    _test_needs_features = [ContextMenuShellExtensionFeature]

    def test_non_vcs_dir_com(self):
        pidl = get_pidl_for_dir(tempfile.tempdir)
        folder = get_isf_for_pidl(pidl)
        # Ask the parent for our IContextMenu.
        inout, cm = folder.GetUIObjectOf(0, [pidl[-1:]],
                                         shell.IID_IContextMenu, 0)
        self._test_non_vcs_dir(cm)

    def test_non_vcs_dir_native(self):
        clsid = get_clsid_for_handler("ContextMenu")
        ob = pythoncom.CoCreateInstance(clsid, None, pythoncom.CLSCTX_ALL,
                                        shell.IID_IShellExtInit)
        pidl = get_pidl_for_dir(tempfile.tempdir)
        ob.Initialize(pidl, None, 0)
        cm = ob.QueryInterface(shell.IID_IContextMenu)
        self._test_non_vcs_dir(cm)

    def _test_non_vcs_dir(self, cm):
        # create a menu and ask for it to be filled.
        items = {} # key by text.
        hmenu = win32gui.CreateMenu()
        try:
            cm.QueryContextMenu(hmenu, 0, 1, -1, 0)
            self.failUnless(win32gui.GetMenuItemCount(hmenu) > 0)
            # build a list of all items on the menu the destroy it.
            for i in xrange(win32gui.GetMenuItemCount(hmenu)):
                buf, extras = win32gui_struct.EmptyMENUITEMINFO()
                win32gui.GetMenuItemInfo(hmenu, i, True, buf)
                info = win32gui_struct.UnpackMENUITEMINFO(buf)
                # text is at index 7
                items[info[7]] = info
        finally:
            win32gui.DestroyMenu(hmenu)
        self.failUnless("Bazaar Chec&kout/Branch..." in items, items)
        # the sub-menu
        self.failUnless("Tortoise Bazaar" in items)

# run these via the tbzrcache selftests
