# Copyright (C) 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os

import pythoncom
from win32com.shell import shell, shellcon

from bzrlib.revisionspec import RevisionSpec
from bzrlib.tests import TestCaseWithTransport, TestCaseInTempDir

from tbzrlib.tests.shellext import get_clsid_for_handler, _ShellExtensionFeature

from test_iconoverlay import get_overlay_id_for_state

# ICopyHookW didn't appear until pywin32-213
try:
    IID_ICopyHook = shell.IID_ICopyHookW
except AttributeError:
    IID_ICopyHook = shell.IID_ICopyHook


CopyHookShellExtensionFeature = _ShellExtensionFeature('CopyHook')

class TestCopyHook(TestCaseWithTransport):
    _test_needs_features = [CopyHookShellExtensionFeature]

    def test_instantiates(self):
        clsid = get_clsid_for_handler('CopyHook')
        ob = pythoncom.CoCreateInstance(clsid, None, pythoncom.CLSCTX_ALL,
                                        pythoncom.IID_IUnknown)
        ob.QueryInterface(IID_ICopyHook)

    def test_simple_call(self):
        clsid = get_clsid_for_handler('CopyHook')
        ob = pythoncom.CoCreateInstance(clsid, None, pythoncom.CLSCTX_ALL,
                                        IID_ICopyHook)
        ob.CopyCallback(0, shellcon.FO_MOVE, 0, "source.txt", 0, "dest.txt", 0)

    def test_hook(self):
        self.requireFeature(CopyHookShellExtensionFeature)
        tree = self.make_branch_and_tree('tree')
        self.build_tree(['tree/dir/'])

        path = os.path.abspath(os.path.join('tree', u'dir'))
        to_path = os.path.abspath(os.path.join('tree', u'new-dir'))
        # Use our icon overlay handler to fetch the status for a child - that
        # should cause a watcher to be created.
        is_member = get_overlay_id_for_state(0).IsMemberOf(path, 0)==0
        self.failUnless(is_member, "overlay handler doesn't want our file?") 

        # Now perform the shell move - the our handler should be called, which
        # will inturn call the remote process to drop the handle.
        info = (0, shellcon.FO_MOVE, path, to_path, 0, None, None)
        shell.SHFileOperation(info)
        self.failIf(os.path.isdir(path), "shell move appeared to fail")
        self.failUnless(os.path.isdir(to_path), "shell move appeared to fail")
        # that's all folks - if our CopyHook handler didn't work correctly,
        # the operation would have failed due to the directory handle being
        # opened.

# run these via the tbzrcache selftests
