# Copyright (C) 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os

from tbzrlib.actions import get_actions_for_files

# XXX - fix/move this base class?
from tbzrlib.tests.test_wtcache import TestWithSimpleTreeStates

class TestActions(TestWithSimpleTreeStates):
    def assertHasAction(self, actions, aid):
        for a in actions:
            if a is not None and a.id == aid:
                return
        self.fail("Actions %r didn't include %r" % (actions, aid))

    def assertHasNoAction(self, actions, aid):
        for a in actions:
            if a is not None and a.id == aid:
                self.fail("Actions %r should not include %r" % (actions, aid))

    def testActionsAdded(self):
        tree, root_path = self.get_tree_with_added()
        main, sub = get_actions_for_files([unicode("tree/dir/a")])
        self.assertHasAction(main, 'tbzr.commit')
        self.assertHasAction(sub, 'tbzr.diff')

    def testActionsUnchanged(self):
        tree, root_path = self.get_tree_with_unchanged()
        main, sub = get_actions_for_files([unicode("tree/dir/a")])
        # Like TSVN, We still offer 'commit' for unmodified items...
        self.assertHasAction(main, 'tbzr.commit')
        # bit not 'diff'
        self.assertHasNoAction(sub, 'tbzr.diff')

    def testActionsDeleted(self):
        tree, root_path = self.get_tree_with_deleted()
        main, sub = get_actions_for_files([unicode("tree/dir/a")])
        self.assertHasAction(main, 'tbzr.commit')
        self.assertHasAction(sub, 'tbzr.diff')

    def testActionsModified(self):
        tree, root_path = self.get_tree_with_modified()
        main, sub = get_actions_for_files([unicode("tree/dir/a")])
        self.assertHasAction(main, 'tbzr.commit')
        self.assertHasAction(sub, 'tbzr.diff')

# test can be run directly or via 'tbzrcache selftest [pattern]'    
if __name__=='__main__':
    import unittest
    unittest.main()
