# Copyright (C) 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import struct

from tbzrlib.dispatcher import pack_request_data, unpack_request_data
from tbzrlib import dispatcher
from tbzrlib import app
# XXX - fix/move this base class?
from tbzrlib.tests.test_wtcache import TestWithSimpleTreeStates

from bzrlib.tests import TestCase, TestCaseWithTransport

def pack_complete_request(req_type, req_body):
    # pack up the request header
    fmt = "HBBH"
    ver_maj, ver_min = (0,1)
    total_size = struct.calcsize(fmt) + len(req_body)
    req_header = struct.pack(fmt, total_size, ver_maj, ver_min, req_type)
    return req_header + req_body

def unpack_complete_response(buf, *ret_desc):
    int_size = struct.calcsize("H")
    resp_len, = struct.unpack("H", buf[:int_size])
    assert resp_len==len(buf), \
           "got %d, but response said %d" % (len(buf), resp_len)
    return unpack_request_data(buf[int_size:], *ret_desc)

class TestWireProtocol(TestCase):
    def testUnicode(self):
        buf = pack_request_data(u"foo")
        back = unpack_request_data(buf, unicode)
        self.assertEqual(back, (u"foo",))

    def testMultiple(self):
        buf = pack_request_data(u"foo", 0, u"bar")
        back = unpack_request_data(buf, unicode, int, unicode)
        self.assertEqual(back, (u"foo", 0, u"bar"))

    def testListOfStrings(self):
        data = [(u"Foo",), (u"Bar",), (u"etc",)]
        buf = pack_request_data(data)
        back = unpack_request_data(buf, [(unicode,)])
        self.assertEqual(back, (data,))

    def testListOfStructs(self):
        data = [(u"Foo", 0), (u"Bar", 1)]
        buf = pack_request_data(data)
        back = unpack_request_data(buf, [(unicode, int)])
        self.assertEqual(back, (data,))


class TestFileStatus(TestWithSimpleTreeStates):
    def testFileStatusAdded(self):
        tree, root_path = self.get_tree_with_added()
        d = dispatcher.CommandDispatcher()

        buf = pack_request_data(unicode("tree/dir/a"), 0)
        buf = pack_complete_request(dispatcher.REQUEST_FILE_STATUS, buf)
        ret_buf = d.dispatch(buf)
        (ret,) = unpack_request_data(ret_buf, int)
        self.assertEqual(ret, app.BZRStatus.STATE_ADDED)

class TestMenuCommands(TestWithSimpleTreeStates):
    def testMenuStatusAdded(self):
        tree, root_path = self.get_tree_with_added()
        d = dispatcher.CommandDispatcher()

        buf = pack_request_data([(unicode("tree/dir/a"),)], 0)
        buf = pack_complete_request(dispatcher.REQUEST_MENU_COMMANDS, buf)
        ret_buf = d.dispatch(buf)
        command_desc = [(int, int, unicode)]
        (ret,) = unpack_request_data(ret_buf, [(int, int, unicode, unicode, unicode, command_desc)])
        # check we can enumerate the result like the client tries to.
        for item in ret:
            flags, level, text, long_text, ico, commands = item



# test can be run directly or via 'tbzrcache selftest [pattern]'    
if __name__=='__main__':
    import unittest
    unittest.main()
