# Copyright (C) 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os
import tempfile

from bzrlib.tests import TestCase
from tbzrlib.vcspath import VCSPath

# These tests stolen directly from TSVN.  Note we did not carry all of the
# TSVNPath concepts into this code, so some make no sense.
class TestTSVNTests(TestCase):
    def test_get_directory(self):
        # // Bit tricky, this test, because we need to know something about the file
        # // layout on the machine which is running the test
        import win32api
        windir = win32api.GetWindowsDirectory().decode(sys.getfilesystemencoding())

        # TSVN did 'SetFromUnknown' - we don't have that concept.
        test_path = VCSPath(os.path.join(windir, "win.ini"))
        self.failIf(test_path.is_directory)
        # ATLASSERT(testPath.GetDirectory().GetWinPathString() == sWinDir);
        self.failUnlessEqual(test_path.containing_directory.path, windir)

        # // Now do the test on the win directory itself - It's hard to be sure about the containing directory
        # // but we know it must be different to the directory itself
        test_path = VCSPath(windir)
        self.failUnless(test_path.is_directory)
        self.failUnlessEqual(test_path.directory.path, windir)
        self.failIfEqual(test_path.containing_directory.path, windir)
        self.failUnless(len(test_path.containing_directory.path) < len(windir))

        # // Try a root path
        test_path = VCSPath(u"C:\\")
        self.failUnless(test_path.is_directory)
        #ATLASSERT(testPath.GetDirectory().GetWinPathString().CompareNoCase(_T("C:\\"))==0);

    def todo_admin_dir(self):
        old = """
            CTSVNPath testPath;
            testPath.SetFromUnknown(_T("c:\\.svndir"));
            ATLASSERT(!testPath.IsAdminDir());
            testPath.SetFromUnknown(_T("c:\\test.svn"));
            ATLASSERT(!testPath.IsAdminDir());
            testPath.SetFromUnknown(_T("c:\\.svn"));
            ATLASSERT(testPath.IsAdminDir());
            testPath.SetFromUnknown(_T("c:\\.svndir\\test"));
            ATLASSERT(!testPath.IsAdminDir());
            testPath.SetFromUnknown(_T("c:\\.svn\\test"));
            ATLASSERT(testPath.IsAdminDir());
            
            CTSVNPathList pathList;
            pathList.AddPath(CTSVNPath(_T("c:\\.svndir")));
            pathList.AddPath(CTSVNPath(_T("c:\\.svn")));
            pathList.AddPath(CTSVNPath(_T("c:\\.svn\\test")));
            pathList.AddPath(CTSVNPath(_T("c:\\test")));
            pathList.RemoveAdminPaths();
            ATLASSERT(pathList.GetCount()==2);
            pathList.Clear();
            pathList.AddPath(CTSVNPath(_T("c:\\test")));
            pathList.RemoveAdminPaths();
            ATLASSERT(pathList.GetCount()==1);
        """

    # SortTest skipped - we just use Python sort and lists.

    # RawAppendTest and PathAppendTest skipped - we just use 'join'

    # RemoveDuplicatesTest, RemoveChildrenTest and ListLoadingTest tests skipped
    # - we just use Python list semantics

    def todo_list_loading(self):
        old = """
            CString sPathList(_T("Path1*c:\\path2 with spaces and stuff*\\funnypath\\*"));
            CTSVNPathList testList;
            testList.LoadFromAsteriskSeparatedString(sPathList);
    
            ATLASSERT(testList.GetCount() == 3);
            ATLASSERT(testList[0].GetWinPathString() == _T("Path1"));
            ATLASSERT(testList[1].GetWinPathString() == _T("c:\\path2 with spaces and stuff"));
            ATLASSERT(testList[2].GetWinPathString() == _T("\\funnypath"));
            
            ATLASSERT(testList.GetCommonRoot().GetWinPathString() == _T(""));
            testList.Clear();
            sPathList = _T("c:\\path2 with spaces and stuff*c:\\funnypath\\*");
            testList.LoadFromAsteriskSeparatedString(sPathList);
            ATLASSERT(testList.GetCommonRoot().GetWinPathString() == _T("c:\\"));
        """

    def test_containing_directory(self):
        test_path = VCSPath(u"c:\\a\\b\\c\\d\\e")
        d = test_path.containing_directory
        self.failUnlessEqual(d.path, u"c:\\a\\b\\c\\d")
        d = d.containing_directory
        self.failUnlessEqual(d.path, u"c:\\a\\b\\c")
        d = d.containing_directory
        self.failUnlessEqual(d.path, u"c:\\a\\b")
        d = d.containing_directory
        self.failUnlessEqual(d.path, u"c:\\a")
        d = d.containing_directory
        self.failUnlessEqual(d.path, u"c:\\")
        d = d.containing_directory
        self.failUnless(d is None, d)

    def test_ancestor(self):
        test_path = VCSPath(u"c:\\windows")
        self.failUnlessEqual(test_path.is_ancestor_of(VCSPath(u"c:\\")), False)
        self.failUnlessEqual(test_path.is_ancestor_of(VCSPath(u"c:\\windows")), True)
        self.failUnlessEqual(test_path.is_ancestor_of(VCSPath(u"c:\\windowsdummy")), False)
        self.failUnlessEqual(test_path.is_ancestor_of(VCSPath(u"c:\\windows\\test.txt")), True)
        self.failUnlessEqual(test_path.is_ancestor_of(VCSPath(u"c:\\windows\\system32\\test.txt")), True)

    def test_longest_shared_path(self):
        d = VCSPath.longest_shared_path(VCSPath(u"c:\\a\\b"), VCSPath(u"c:\\a\\b"))
        self.failUnlessEqual(d.path, u"c:\\a\\b")
        d = VCSPath.longest_shared_path(VCSPath(u"c:\\a\\ba"), VCSPath(u"c:\\a\\bb"))
        self.failUnlessEqual(d.path, u"c:\\a")
        d = VCSPath.longest_shared_path(VCSPath(u"c:\\a\\b"), VCSPath(u"c:\\a"))
        self.failUnlessEqual(d.path, u"c:\\a")
        d = VCSPath.longest_shared_path(VCSPath(u"c:\\a"), VCSPath(u"c:\\a\\b"))
        self.failUnlessEqual(d.path, u"c:\\a")
        d = VCSPath.longest_shared_path(VCSPath(u"c:\\a"), VCSPath(u"d:\\a"))
        self.failUnlessEqual(d, None)

    # SubversionPathTest skipped - it handles 'http://' etc "paths" and we don't

    def todo_common_root(self):
        old = """
            CTSVNPath pathA (_T("C:\\Development\\LogDlg.cpp"));
            CTSVNPath pathB (_T("C:\\Development\\LogDlg.h"));
            CTSVNPath pathC (_T("C:\\Development\\SomeDir\\LogDlg.h"));
            
            CTSVNPathList list;
            list.AddPath(pathA);
            ATLASSERT(list.GetCommonRoot().GetWinPathString().CompareNoCase(_T("C:\\Development\\LogDlg.cpp"))==0);
            list.AddPath(pathB);
            ATLASSERT(list.GetCommonRoot().GetWinPathString().CompareNoCase(_T("C:\\Development"))==0);
            list.AddPath(pathC);
            ATLASSERT(list.GetCommonRoot().GetWinPathString().CompareNoCase(_T("C:\\Development"))==0);
        """

    def todo_valid_path_and_url(self):
        old = """
            CTSVNPath testPath;
            testPath.SetFromWin(_T("c:\\a\\b\\c.test.txt"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("D:\\.Net\\SpindleSearch\\"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\test folder\\file"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\folder\\"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\ext.ext.ext\\ext.ext.ext.ext"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\.svn"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\com\\file"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\test\\conf"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\LPT"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\test\\LPT"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\com1test"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("\\\\?\\c:\\test\\com1test"));
            ATLASSERT(testPath.IsValidOnWindows());
    
            testPath.SetFromWin(_T("\\\\Share\\filename"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("\\\\Share\\filename.extension"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("\\\\Share\\.svn"));
            ATLASSERT(testPath.IsValidOnWindows());
    
            // now the negative tests
            testPath.SetFromWin(_T("c:\\test:folder"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\file<name"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\something*else"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\folder\\file?nofile"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\ext.>ension"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\com1\\filename"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\com1"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("c:\\com1\\AuX"));
            ATLASSERT(!testPath.IsValidOnWindows());
    
            testPath.SetFromWin(_T("\\\\Share\\lpt9\\filename"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("\\\\Share\\prn"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromWin(_T("\\\\Share\\NUL"));
            ATLASSERT(!testPath.IsValidOnWindows());
            
            // now come some URL tests
            testPath.SetFromSVN(_T("http://myserver.com/repos/trunk"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("https://myserver.com/repos/trunk/file%20with%20spaces"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("svn://myserver.com/repos/trunk/file with spaces"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("svn+ssh://www.myserver.com/repos/trunk"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("http://localhost:90/repos/trunk"));
            ATLASSERT(testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("file:///C:/SVNRepos/Tester/Proj1/tags/t2"));
            ATLASSERT(testPath.IsValidOnWindows());
            // and some negative URL tests
            testPath.SetFromSVN(_T("httpp://myserver.com/repos/trunk"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("https://myserver.com/rep:os/trunk/file%20with%20spaces"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("svn://myserver.com/rep<os/trunk/file with spaces"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("svn+ssh://www.myserver.com/repos/trunk/prn/"));
            ATLASSERT(!testPath.IsValidOnWindows());
            testPath.SetFromSVN(_T("http://localhost:90/repos/trunk/com1"));
            ATLASSERT(!testPath.IsValidOnWindows());
        """

# Tests we have invented
class TestTVCSTests(TestCase):
    def test_slashes(self):
        tempdir = tempfile.tempdir.decode(sys.getfilesystemencoding())
        tempdir2 = tempdir + os.path.sep
        self.failUnlessEqual(VCSPath(tempdir), VCSPath(tempdir2))

if __name__=='__main__':
    # XXX - how to hook into bzrlib selftest?
    import unittest
    unittest.main()
