# This is a package.
from Queue import Queue, Empty
import threading
import logging

from tbzrlib.vcspath import VCSPath

from win32watcher import DirectoryWatcher as DirectoryWatcherBase

logger = logging.getLogger('bzr.tbzr.dirwatcher')

# A function called in a b/g thread to process change events.
def process_change_events(watcher, cache):
    while not watcher.stopping:
        path = watcher.change_queue.get()
        if watcher.stopping or path is None:
            break
        cache.update_cache_for_path(VCSPath(path).directory)
    logger.debug("change collector thread stopping")

class DirectoryWatcher(DirectoryWatcherBase):
    def __init__(self, status_cache, hwnd=None):
        self.stopping = False
        self.change_queue = Queue()
        self.change_thread = threading.Thread(target=process_change_events,
                                              args=(self, status_cache))
        self.change_thread.daemon = True
        self.change_thread.start()
        DirectoryWatcherBase.__init__(self, hwnd)

    def run(self):
        try:
            try:
                DirectoryWatcherBase.run(self)
            finally:
                # We must be stopping - stop our queue collector.
                self.stopping = True
                self._flush_queue()
                self.change_queue.put(None)
                self.change_thread.join(5)
                if self.change_thread.isAlive():
                    logger.warning("change collector thread failed to stop")
        except:
            logger.exception("Watcher thread failed")

    def _flush_queue(self):
        while True:
            try:
                self.change_queue.get_nowait()
            except Empty:
                break

    def stop(self):
        self.stopping = True
        self._flush_queue()
        DirectoryWatcherBase.stop(self)

    def on_change_detected(self, filename, action):
        #Ignore ADDED. (To avoid updating while editing name of new file)
        if action == 1: 
            return
        self.change_queue.put(filename)
